#!/bin/bash

# TODO: Comments and some more user output

opt_edition="$1"

set -o nounset # exit script if an unitilialized variable is used
set -o errexit # exit script if any statement returns a non-true value

KT_COMMUNITY="knowledgetree-ce"
KT_CERTIFIED="knowledgetree"

# Check that we are run as root.
# Return 0 if yes, or 1 if not or undeterminable.
function is_root() {
  uid=$(id -u 2> /dev/null)
  
  if [ -n "$uid" ]; then
    if [ "$uid" != "0" ]; then
      echo "FAILED: You need root privileges to run this script.";
      echo
      return 1
    fi
  else
    echo "Could not detect UID";
    return 1
  fi
}

# Pretty print a message saying that the distro is unsupported.
function unsupported_distro() {
  echo
  echo "Unable to install: Your distribution is not suitable for installation using"
  echo "KnowledgeTree's DEB/RPM repositories. You can install KnowledgeTree Community"
  echo "Edition on most Linux distributions using the generic tarball installer. For"
  echo "more information, see http://www.Knowledgetree.com/community/knowledgetree-ce"
  echo
}

# Check that distro is supported and get package management system type.
# Print unsupported distro and return 1 if distro is not supported.
function distro_info() {
  distro=$(lsb_release --short --id)
  
  # TODO: verify the lsb_release output of Debian, CentOS, Fedora, RedHatEnterpriseServer
  case "$distro" in
    "Ubuntu"|"Debian")
      package_type="deb"
      ;;
    "CentOS"|"Fedora"|"RedHatEnterpriseServer")
      package_type="rpm"
      ;;
    * )
      unsupported_distro
      return 1
      ;;
  esac
}

# Prompt user for which version of KT to install
function get_meta_packge() {
  edition="na"
  
  echo "Choose the KnowledgeTree edition that you want to install"
  echo "========================================================="
  echo
  echo -e "\t[1]: Community edition"
  echo -e "\t[2]: Certified edition"
  echo
  echo -n "Which edition would you like to install (1 or 2)? "
  read edition
  
  until [[ "$edition" = "1" || "$edition" = "2" ]] ; do
    echo -n "Please type either 1 for Community edition or 2 for Certified edition: "
    read edition
  done
  
  case "$edition" in
    "1" )
      meta_package_name=$KT_COMMUNITY
      ;;
    "2" )
      meta_package_name=$KT_CERTIFIED
      ;;
   esac
}

# Print usage if wrong args
function usage() {
  echo
  echo "${0} [ce|co]"
  echo -e "\tco - Certified edition"
  echo -e "\tce - Community edition"
  echo
}

function configure_rpm_repos() {
# TODO: We might want to test if the file exists already
# and if it does, don't overwrite it
(
cat <<'EOF'
[KT]
name=KT
baseurl=http://repos.knowledgetree.com/rpm/rhel/5/$basearch
enabled=1
gpgcheck=0

[KTnoarch]
name=KT - noarch
baseurl=http://repos.knowledgetree.com/rpm/rhel/5/noarch
enabled=1
gpgcheck=0

[Zend]
name=Zend Server 
baseurl=http://repos.zend.com/zend-server/rpm/$basearch
enabled=1
gpgcheck=0

[Zend_noarch]
name=Zend Server - noarch
baseurl=http://repos.zend.com/zend-server/rpm/noarch
enabled=1
gpgcheck=0
EOF
) > "/tmp/knowledgetree.repo"

  mv /tmp/knowledgetree.repo /etc/yum.repos.d/knowledgetree.repo
  if [ $? -ne 0 ]; then
    echo "Failed to create Yum repo file"
    return 1
  fi
  
  if yum clean all ; then
    return 0
  else
    echo "Failed to update Yum."
    return 1
  fi
}

function configure_deb_repos() {
# TODO: We might want to test if the file exists already
# and if it does, don't overwrite it
(
cat <<'EOF'
# KnowledgeTree
deb http://repos.knowledgetree.com/deb/knowledgetree knowledgetree-ce main
deb http://repos.knowledgetree.com/deb/knowledgetree knowledgetree non-free
# Zend
deb http://repos.zend.com/zend-server/deb server non-free
EOF
) > "/tmp/knowledgetree.list"
  
  mv /tmp/knowledgetree.list /etc/apt/sources.list.d/knowledgetree.list
  if [ $? -ne 0 ]; then
    echo "Failed to create sources list file"
    return 1
  fi
  
  # TODO: Split this pipe up and test exit status of each command
  wget http://repos.zend.com/deb/zend.key -O- | apt-key add -
  wget http://repos.knowledgetree.com/deb/knowledgetree/kt.key -O- | apt-key add -
  
  # TODO: Enable test when Zend fixes their repo
  # if apt-get update ; then
  #   return 0
  # else
  #   echo
  #   echo "Failed to update Apt repositories"
  #   echo
  #   return 1
  # fi
  apt-get update
  return 0
}

# Call appropriate repo configure and install command
# Returns 0 on success else 1
function install_kt() {
  package_type=$1
  meta_package_name=$2
  
  case "$package_type" in
    "rpm" )
      if configure_rpm_repos ; then
        install_cmd="yum install"
      else
        return 1
      fi
      ;;
    "deb" )
      if configure_deb_repos ; then
        install_cmd="apt-get install"
      else
        return 1
      fi
      ;;
  esac
  
  $install_cmd $meta_package_name
}


## M A I N ##
echo
echo "Running this script will preform the following:"
echo "* Configure your package manager to use the KnowledgeTree repository"
echo "* Install KnowledgeTree on your system using your package manager"
echo
echo "**PLEASE NOTE**"
echo
echo "This install will not work if your current KnowledgeTree 3.6.1 Stack is running at port 80."
echo "If you have an existing KnowledgeTree installation please ensure you shutdown down all services now by running:"
echo "/path/to/ktdms/dmsctl.sh stop"
echo
echo "Hit Ctrl+C to abort now in order to shutdown your services, or ENTER if you are ready to install KnowledgeTree."
echo
read

# make sure we are root
is_root

# get supported distro info and fail if distro is unsupported
distro_info
opt_edition="co"
if [ -z "$opt_edition" ]; then
  get_meta_packge
else
  case "$opt_edition" in
    "co" )
      meta_package_name=$KT_CERTIFIED
      installed=$?
      ;;
    "ce" )
      meta_package_name=$KT_COMMUNITY
      installed=$?
      ;;
    * )
      usage
      exit 1
      ;;
  esac
fi

if install_kt $package_type $meta_package_name ; then
  echo
  echo "KnowledgeTree was successfully installed."
  echo
else
  echo
  echo "Installation was not completed. See output above for detailed error information."
  echo
fi
